@@ -21,8 +21,9 @@ import typing as _typing
 
 from optax import assignment
 from optax import contrib
+from optax import experimental
 from optax import losses
-from optax import monte_carlo
+from optax import microbatching
 from optax import perturbations
 from optax import projections
 from optax import schedules
@@ -59,6 +60,7 @@ from optax._src.alias import rmsprop
 from optax._src.alias import rprop
 from optax._src.alias import sgd
 from optax._src.alias import sign_sgd
+from optax._src.alias import signum
 from optax._src.alias import sm3
 from optax._src.alias import yogi
 from optax._src.base import EmptyState
@@ -88,6 +90,7 @@ from optax._src.linesearch import scale_by_backtracking_linesearch
 from optax._src.linesearch import scale_by_zoom_linesearch
 from optax._src.linesearch import ScaleByBacktrackingLinesearchState
 from optax._src.linesearch import ScaleByZoomLinesearchState
+from optax._src.linesearch import value_and_grad_from_state
 from optax._src.linesearch import ZoomLinesearchInfo
 from optax._src.lookahead import lookahead
 from optax._src.lookahead import LookaheadParams
@@ -145,7 +148,6 @@ from optax._src.update import incremental_update
 from optax._src.update import periodic_update
 from optax._src.utils import multi_normal
 from optax._src.utils import scale_gradient
-from optax._src.utils import value_and_grad_from_state
 
 # TODO(mtthss): remove contrib aliases from flat namespace once users updated.
 # Deprecated modules
@@ -157,6 +159,7 @@ from optax.contrib import dpsgd as _deprecated_dpsgd
 
 
 # TODO(mtthss): remove aliases after updates.
+microbatch = microbatching.microbatch
 adaptive_grad_clip = transforms.adaptive_grad_clip
 AdaptiveGradClipState = EmptyState
 clip = transforms.clip
@@ -198,6 +201,9 @@ selective_transform = transforms.selective_transform
 masked = transforms.masked
 MaskedNode = transforms.MaskedNode
 MaskedState = transforms.MaskedState
+measure_with_ema = transforms.measure_with_ema
+monitor = transforms.monitor
+MonitorState = transforms.MonitorState
 MultiSteps = transforms.MultiSteps
 MultiStepsState = transforms.MultiStepsState
 ShouldSkipUpdateFunction = transforms.ShouldSkipUpdateFunction
@@ -240,6 +246,7 @@ cosine_distance = losses.cosine_distance
 cosine_similarity = losses.cosine_similarity
 ctc_loss = losses.ctc_loss
 ctc_loss_with_forward_probs = losses.ctc_loss_with_forward_probs
+generalized_kl_divergence = losses.generalized_kl_divergence
 hinge_loss = losses.hinge_loss
 huber_loss = losses.huber_loss
 kl_divergence = losses.kl_divergence
@@ -281,6 +288,11 @@ _deprecations = {
         ),
         _deprecated_dpsgd,
     ),
+    # Added Nov 2025
+    "monte_carlo": (
+        "optax.monte_carlo is deprecated. (optax v0.2.7 or newer).",
+        None,
+    ),
 }
 # pylint: disable=g-import-not-at-top
 # pylint: disable=g-bad-import-order
@@ -303,7 +315,7 @@ del _typing
 # pylint: enable=g-importing-member
 
 
-__version__ = '0.2.6'
+__version__ = '0.2.7'
 
 __all__ = (
     "adabelief",
@@ -357,6 +369,7 @@ __all__ = (
     "FactoredState",
     "flatten",
     "fromage",
+    "generalized_kl_divergence",
     "global_norm",
     "GradientTransformation",
     "GradientTransformationExtraArgs",
@@ -364,6 +377,7 @@ __all__ = (
     "huber_loss",
     "identity",
     "incremental_update",
+    "init_empty_state",
     "inject_hyperparams",
     "InjectHyperparamsState",
     "join_schedules",
@@ -384,6 +398,11 @@ __all__ = (
     "MaskOrFn",
     "MaskedState",
     "matrix_inverse_pth_root",
+    "measure_with_ema",
+    "microbatch",
+    "monitor",
+    "microbatch",
+    "MonitorState",
     "multi_normal",
     "multi_transform",  # for backwards compatibility
     "MultiSteps",
@@ -468,6 +487,7 @@ __all__ = (
     "ShouldSkipUpdateFunction",
     "sigmoid_binary_cross_entropy",
     "sign_sgd",
+    "signum",
     "skip_large_updates",
     "skip_not_finite",
     "sm3",
