@@ -15,10 +15,9 @@
 """Base interfaces and datatypes."""
 
 from collections.abc import Callable
-from typing import (Any, NamedTuple, Optional, Protocol, Sequence, Union,
-                    runtime_checkable)
+from typing import (Any, Iterable, Mapping, NamedTuple, Optional, Protocol,
+                    Sequence, Union, runtime_checkable)
 
-import chex
 import jax
 import jax.numpy as jnp
 
@@ -29,12 +28,15 @@ NO_PARAMS_MSG = (
 
 PyTree = Any
 Shape = Sequence[int]
+PRNGKey = jax.Array
+ArrayTree = Union[
+    jax.typing.ArrayLike, Iterable['ArrayTree'], Mapping[Any, 'ArrayTree']]
 
-OptState = chex.ArrayTree  # States are arbitrary nests of `jnp.ndarrays`.
-Params = chex.ArrayTree  # Parameters are arbitrary nests of `jnp.ndarrays`.
+OptState = ArrayTree  # States are arbitrary nests of `jnp.ndarrays`.
+Params = ArrayTree  # Parameters are arbitrary nests of `jnp.ndarrays`.
 Updates = Params  # Gradient updates are of the same type as parameters.
 
-Schedule = Callable[[chex.Numeric], chex.Numeric]
+Schedule = Callable[[jax.typing.ArrayLike], jax.typing.ArrayLike]
 ScheduleState = Any
 ScalarOrSchedule = Union[float, jax.Array, Schedule]
 
@@ -57,7 +59,7 @@ class StatefulSchedule(Protocol):
       self,
       state: ScheduleState,
       **extra_args,
-  ) -> chex.Numeric:
+  ) -> jax.typing.ArrayLike:
     """Computes the current schedule value."""
 
 
@@ -90,7 +92,7 @@ class TransformUpdateFn(Protocol):
   access to the current values of the parameters.
 
   For the case where additional arguments are required, an alternative interface
-  may be used, see ``TransformUpdateExtraArgsFn`` for details.
+  may be used, see :py:class:`.TransformUpdateExtraArgsFn` for details.
   """
 
   def __call__(
@@ -295,7 +297,8 @@ def stateless(
 
 
 def stateless_with_tree_map(
-    f: Callable[[chex.Array, Optional[chex.Array]], chex.Array],
+    f: Callable[[jax.typing.ArrayLike, Optional[jax.typing.ArrayLike]],
+                jax.typing.ArrayLike],
 ) -> GradientTransformation:
   """Creates a stateless transformation from an update-like function for arrays.
 
