@@ -15,16 +15,16 @@
 """Tests for base functions in `base.py`."""
 
 from absl.testing import absltest
-import chex
 import jax
 import jax.numpy as jnp
 import numpy as np
 from optax._src import base
+from optax._src import test_utils
 
 # pylint:disable=no-value-for-parameter
 
 
-class BaseTest(chex.TestCase):
+class BaseTest(absltest.TestCase):
 
   def test_typing(self):
     """Ensure that the type annotations work for the update function."""
@@ -44,25 +44,23 @@ class BaseTest(chex.TestCase):
 
     g(f)
 
-  @chex.all_variants
   def test_set_to_zero_returns_tree_of_correct_zero_arrays(self):
     """Tests that zero transform returns a tree of zeros of correct shape."""
     grads = ({'a': np.ones((3, 4)), 'b': 1.0}, np.ones((1, 2, 3)))
-    updates, _ = self.variant(base.set_to_zero().update)(
+    updates, _ = jax.jit(base.set_to_zero().update)(
         grads, base.EmptyState()
     )
     correct_zeros = ({'a': np.zeros((3, 4)), 'b': 0.0}, np.zeros((1, 2, 3)))
-    chex.assert_trees_all_close(updates, correct_zeros, rtol=0)
+    test_utils.assert_trees_all_close(updates, correct_zeros, rtol=0)
 
-  @chex.all_variants(with_pmap=False)
   def test_set_to_zero_is_stateless(self):
     """Tests that the zero transform returns an empty state."""
     self.assertEqual(
-        self.variant(base.set_to_zero().init)(params=None), base.EmptyState()
+        jax.jit(base.set_to_zero().init)(params=None), base.EmptyState()
     )
 
 
-class ExtraArgsTest(chex.TestCase):
+class ExtraArgsTest(absltest.TestCase):
 
   def test_isinstance(self):
     """Locks in behavior for comparing transformations."""
@@ -123,10 +121,9 @@ class ExtraArgsTest(chex.TestCase):
     self.assertEqual(metrics['learning_rate'], 0.3)
 
 
-class StatelessTest(chex.TestCase):
+class StatelessTest(absltest.TestCase):
   """Tests for the stateless transformation."""
 
-  @chex.all_variants
   def test_stateless(self):
     params = {'a': jnp.zeros((1, 2)), 'b': jnp.ones((1,))}
     updates = {'a': jnp.ones((1, 2)), 'b': jnp.full((1,), 2.0)}
@@ -136,12 +133,11 @@ class StatelessTest(chex.TestCase):
       return jax.tree.map(lambda g_, p_: g_ + 0.1 * p_, g, p)
 
     state = opt.init(params)
-    update_fn = self.variant(opt.update)
+    update_fn = jax.jit(opt.update)
     new_updates, _ = update_fn(updates, state, params)
     expected_updates = {'a': jnp.ones((1, 2)), 'b': jnp.array([2.1])}
-    chex.assert_trees_all_close(new_updates, expected_updates)
+    test_utils.assert_trees_all_close(new_updates, expected_updates)
 
-  @chex.all_variants
   def test_stateless_no_params(self):
     updates = {'linear': jnp.full((5, 3), 3.0)}
 
@@ -150,10 +146,10 @@ class StatelessTest(chex.TestCase):
       return jax.tree.map(lambda g_: g_ * 2, g)
 
     state = opt.init(None)  # pytype: disable=wrong-arg-types  # numpy-scalars
-    update_fn = self.variant(opt.update)
+    update_fn = jax.jit(opt.update)
     new_updates, _ = update_fn(updates, state)
     expected_updates = {'linear': jnp.full((5, 3), 6.0)}
-    chex.assert_trees_all_close(new_updates, expected_updates)
+    test_utils.assert_trees_all_close(new_updates, expected_updates)
 
   def test_init_returns_emptystate(self):
     def weight_decay(g, p):
@@ -164,31 +160,29 @@ class StatelessTest(chex.TestCase):
     self.assertIsInstance(state, base.EmptyState)
 
 
-class StatelessWithTreeMapTest(chex.TestCase):
+class StatelessWithTreeMapTest(absltest.TestCase):
   """Tests for the stateless_with_tree_map transformation."""
 
-  @chex.all_variants
   def test_stateless_with_tree_map(self):
     params = {'a': jnp.zeros((1, 2)), 'b': jnp.ones((1,))}
     updates = {'a': jnp.ones((1, 2)), 'b': jnp.full((1,), 2.0)}
 
     opt = base.stateless_with_tree_map(lambda g, p: g + 0.1 * p)
     state = opt.init(params)
-    update_fn = self.variant(opt.update)
+    update_fn = jax.jit(opt.update)
     new_updates, _ = update_fn(updates, state, params)
     expected_updates = {'a': jnp.ones((1, 2)), 'b': jnp.array([2.1])}
-    chex.assert_trees_all_close(new_updates, expected_updates)
+    test_utils.assert_trees_all_close(new_updates, expected_updates)
 
-  @chex.all_variants
   def test_stateless_with_tree_map_no_params(self):
     updates = {'linear': jnp.full((5, 3), 3.0)}
 
     opt = base.stateless_with_tree_map(lambda g, _: g * 2.0)
     state = opt.init(None)  # pytype: disable=wrong-arg-types  # numpy-scalars
-    update_fn = self.variant(opt.update)
+    update_fn = jax.jit(opt.update)
     new_updates, _ = update_fn(updates, state)
     expected_updates = {'linear': jnp.full((5, 3), 6.0)}
-    chex.assert_trees_all_close(new_updates, expected_updates)
+    test_utils.assert_trees_all_close(new_updates, expected_updates)
 
   def test_init_returns_emptystate(self):
     opt = base.stateless_with_tree_map(lambda g, p: g + 0.1 * p)
