@@ -15,9 +15,8 @@
 """Gradient transformations."""
 
 import functools
-from typing import NamedTuple, Optional, Union
+from typing import Literal, NamedTuple, Optional
 
-import chex
 import jax
 from jax import nn
 import jax.numpy as jnp
@@ -44,7 +43,8 @@ class ScaleByRssState(NamedTuple):
 
 
 def scale_by_rss(
-    initial_accumulator_value: float = 0.1, eps: float = 1e-7
+    initial_accumulator_value: jax.typing.ArrayLike = 0.1,
+    eps: jax.typing.ArrayLike = 1e-7
 ) -> base.GradientTransformation:
   """Rescale updates by the root of the sum of all squared gradients to date.
 
@@ -88,14 +88,14 @@ class ScaleByRmsState(NamedTuple):
 class ScaleByRmsWithCountState(NamedTuple):
   """State for exponential root mean-squared (RMS)-normalized updates."""
 
-  count: chex.Array  # shape=(), dtype=jnp.int32.
+  count: jax.typing.ArrayLike  # shape=(), dtype=jnp.int32.
   nu: base.Updates
 
 
 def scale_by_rms(
-    decay: float = 0.9,
-    eps: float = 1e-8,
-    initial_scale: float = 0.0,
+    decay: jax.typing.ArrayLike = 0.9,
+    eps: jax.typing.ArrayLike = 1e-8,
+    initial_scale: jax.typing.ArrayLike = 0.0,
     eps_in_sqrt: bool = True,
     bias_correction: bool = False,
 ) -> base.GradientTransformation:
@@ -162,15 +162,15 @@ class ScaleByRStdDevState(NamedTuple):
 class ScaleByRStdDevWithCountState(NamedTuple):
   """State for centered exponential moving average of squares of updates."""
 
-  count: chex.Array  # shape=(), dtype=jnp.int32.
+  count: jax.typing.ArrayLike  # shape=(), dtype=jnp.int32.
   mu: base.Updates
   nu: base.Updates
 
 
 def scale_by_stddev(
-    decay: float = 0.9,
-    eps: float = 1e-8,
-    initial_scale: float = 0.0,
+    decay: jax.typing.ArrayLike = 0.9,
+    eps: jax.typing.ArrayLike = 1e-8,
+    initial_scale: jax.typing.ArrayLike = 0.0,
     eps_in_sqrt: bool = True,
     bias_correction: bool = False,
 ) -> base.GradientTransformation:
@@ -238,17 +238,17 @@ def scale_by_stddev(
 class ScaleByAdamState(NamedTuple):
   """State for the Adam algorithm."""
 
-  count: chex.Array  # shape=(), dtype=jnp.int32.
+  count: jax.typing.ArrayLike  # shape=(), dtype=jnp.int32.
   mu: base.Updates
   nu: base.Updates
 
 
 def scale_by_adam(
-    b1: float = 0.9,
-    b2: float = 0.999,
-    eps: float = 1e-8,
-    eps_root: float = 0.0,
-    mu_dtype: Optional[chex.ArrayDType] = None,
+    b1: jax.typing.ArrayLike = 0.9,
+    b2: jax.typing.ArrayLike = 0.999,
+    eps: jax.typing.ArrayLike = 1e-8,
+    eps_root: jax.typing.ArrayLike = 0.0,
+    mu_dtype: Optional[jax.typing.DTypeLike] = None,
     *,
     nesterov: bool = False,
 ) -> base.GradientTransformation:
@@ -303,6 +303,7 @@ def scale_by_adam(
         is_leaf=lambda x: x is None,
     )
     mu = optax.tree.cast(mu, mu_dtype)
+    nu = optax.tree.cast_like(nu, state.nu)
     return updates, ScaleByAdamState(count=count_inc, mu=mu, nu=nu)
 
   return base.GradientTransformation(init_fn, update_fn)
@@ -311,18 +312,20 @@ def scale_by_adam(
 class ScaleByAmsgradState(NamedTuple):
   """State for the AMSGrad algorithm."""
 
-  count: chex.Array  # shape=(), dtype=jnp.int32.
+  count: jax.typing.ArrayLike  # shape=(), dtype=jnp.int32.
   mu: base.Updates
   nu: base.Updates
   nu_max: base.Updates
 
 
 def scale_by_amsgrad(
-    b1: float = 0.9,
-    b2: float = 0.999,
-    eps: float = 1e-8,
-    eps_root: float = 0.0,
-    mu_dtype: Optional[chex.ArrayDType] = None,
+    b1: jax.typing.ArrayLike = 0.9,
+    b2: jax.typing.ArrayLike = 0.999,
+    eps: jax.typing.ArrayLike = 1e-8,
+    eps_root: jax.typing.ArrayLike = 0.0,
+    mu_dtype: Optional[jax.typing.DTypeLike] = None,
+    bias_correction_mu: bool = True,
+    bias_correction_nu: bool = True,
 ) -> base.GradientTransformation:
   """Rescale updates according to the AMSGrad algorithm.
 
@@ -336,6 +339,11 @@ def scale_by_amsgrad(
       numerical stability when backpropagating gradients through the rescaling.
     mu_dtype: Optional `dtype` to be used for the first order accumulator; if
       `None` then the `dtype` is inferred from `params` and `updates`.
+    bias_correction_mu: Whether to apply bias correction to the first moment
+      estimate. Set to ``False`` to match the original AMSGrad paper.
+    bias_correction_nu: Whether to apply bias correction to the second moment
+      estimate before taking the elementwise maximum (``nu_max``). Set to
+      ``False`` to match the original AMSGrad paper.
 
   Returns:
     A :class:`optax.GradientTransformation` object.
@@ -356,9 +364,18 @@ def scale_by_amsgrad(
     mu = optax.tree.update_moment(updates, state.mu, b1, 1)
     nu = optax.tree.update_moment_per_elem_norm(updates, state.nu, b2, 2)
     count_inc = numerics.safe_increment(state.count)
-    mu_hat = optax.tree.bias_correction(mu, b1, count_inc)
-    nu_hat = optax.tree.bias_correction(nu, b2, count_inc)
-    nu_max = jax.tree.map(jnp.maximum, state.nu_max, nu_hat)
+
+    if bias_correction_mu:
+      mu_hat = optax.tree.bias_correction(mu, b1, count_inc)
+    else:
+      mu_hat = mu
+
+    if bias_correction_nu:
+      nu_eff = optax.tree.bias_correction(nu, b2, count_inc)
+    else:
+      nu_eff = nu
+
+    nu_max = jax.tree.map(jnp.maximum, state.nu_max, nu_eff)
     updates = jax.tree.map(
         lambda m, v: None if m is None else m / (jnp.sqrt(v + eps_root) + eps),
         mu_hat,
@@ -374,7 +391,9 @@ def scale_by_amsgrad(
 
 
 def scale_by_adamax(
-    b1: float = 0.9, b2: float = 0.999, eps: float = 1e-8
+    b1: jax.typing.ArrayLike = 0.9,
+    b2: jax.typing.ArrayLike = 0.999,
+    eps: jax.typing.ArrayLike = 1e-8
 ) -> base.GradientTransformation:
   """Rescale updates according to the Adamax algorithm.
 
@@ -410,14 +429,17 @@ def scale_by_adamax(
 class ScaleByLionState(NamedTuple):
   """State for the Lion algorithm."""
 
-  count: chex.Array  # shape=(), dtype=jnp.int32.
+  count: jax.typing.ArrayLike  # shape=(), dtype=jnp.int32.
   mu: base.Updates
 
 
 def scale_by_lion(
-    b1: float = 0.9,
-    b2: float = 0.99,
-    mu_dtype: Optional[chex.ArrayDType] = None,
+    b1: jax.typing.ArrayLike = 0.9,
+    b2: jax.typing.ArrayLike = 0.99,
+    mu_dtype: Optional[jax.typing.DTypeLike] = None,
+    *,
+    mode: Literal['hard', 'smooth', 'refined'] = 'hard',
+    smooth_beta: float = 1.0,
 ) -> base.GradientTransformation:
   """Rescale updates according to the Lion algorithm.
 
@@ -428,6 +450,9 @@ def scale_by_lion(
     b2: Decay rate for the exponentially weighted average of grads.
     mu_dtype: Optional `dtype` to be used for the momentum; if `None` then the
       `dtype is inferred from `params` and `updates`.
+    mode : Which sign variant to use: "Hard (sign)", "smooth (tanh smoothing),"
+      or "refined" (linear around 0, saturate to sign for a large value).
+    smooth_beta: Smoothing factor used when mode == "smooth"
 
   Returns:
     A :class:`optax.GradientTransformation` object.
@@ -440,10 +465,25 @@ def scale_by_lion(
     return ScaleByLionState(count=jnp.zeros([], jnp.int32), mu=mu)
 
   def update_fn(updates, state, params=None):
+
     del params
-    updates_new = jax.tree.map(
-        lambda g, m: jnp.sign((1.0 - b1) * g + b1 * m), updates, state.mu
-    )
+
+    def _comb(g, m):
+      x = (1.0 - b1) * g + b1 * m
+      if mode == 'hard':
+        return jnp.sign(x)
+      elif mode == 'smooth':
+        return jnp.tanh(smooth_beta * x)
+      elif mode == 'refined':
+        # Keep small values linear, saturate to sign for large values.
+        return jnp.where(jnp.abs(x) < 1.0, x, jnp.sign(x))
+      else:  # pytype: disable=unreachable
+        raise ValueError(
+            f'Unknown lion mode: {mode}. '
+            'It needs to be one of ["hard", "smooth", "refined"].'
+        )
+
+    updates_new = jax.tree.map(_comb, updates, state.mu)
     mu = optax.tree.update_moment(updates, state.mu, b2, 1)
     mu = optax.tree.cast(mu, mu_dtype)
     count_inc = numerics.safe_increment(state.count)
@@ -452,7 +492,7 @@ def scale_by_lion(
   return base.GradientTransformation(init_fn, update_fn)
 
 
-def scale(step_size: float) -> base.GradientTransformation:
+def scale(step_size: jax.typing.ArrayLike) -> base.GradientTransformation:
   """Scale updates by some fixed scalar `step_size`.
 
   Args:
@@ -471,7 +511,7 @@ def scale(step_size: float) -> base.GradientTransformation:
 
 
 def scale_by_param_block_norm(
-    min_scale: float = 1e-3,
+    min_scale: jax.typing.ArrayLike = 1e-3,
 ) -> base.GradientTransformation:
   """Scale updates for each param block by the norm of that block's parameters.
 
@@ -497,7 +537,7 @@ def scale_by_param_block_norm(
 
 
 def scale_by_param_block_rms(
-    min_scale: float = 1e-3,
+    min_scale: jax.typing.ArrayLike = 1e-3,
 ) -> base.GradientTransformation:
   """Scale updates by rms of the gradient for each param vector or matrix.
 
@@ -532,7 +572,7 @@ class ScaleByAdaDeltaState(NamedTuple):
 
 
 def scale_by_adadelta(
-    rho: float = 0.9, eps: float = 1e-6
+    rho: jax.typing.ArrayLike = 0.9, eps: jax.typing.ArrayLike = 1e-6
 ) -> base.GradientTransformation:
   """Rescale updates according to the Adadelta algorithm.
 
@@ -575,15 +615,15 @@ class ScaleByAdanState(NamedTuple):
   v: base.Updates
   n: base.Updates
   g: base.Updates
-  t: chex.Array
+  t: jax.typing.ArrayLike
 
 
 def scale_by_adan(
-    b1: float = 0.98,
-    b2: float = 0.92,
-    b3: float = 0.99,
-    eps: float = 1e-8,
-    eps_root: float = 0.0,
+    b1: jax.typing.ArrayLike = 0.98,
+    b2: jax.typing.ArrayLike = 0.92,
+    b3: jax.typing.ArrayLike = 0.99,
+    eps: jax.typing.ArrayLike = 1e-8,
+    eps_root: jax.typing.ArrayLike = 0.0,
 ) -> base.GradientTransformation:
   """Rescale updates according to the Adan algorithm.
 
@@ -651,16 +691,16 @@ def scale_by_adan(
 class ScaleByBeliefState(NamedTuple):
   """State for the rescaling by AdaBelief algorithm."""
 
-  count: chex.Array  # shape=(), dtype=jnp.int32.
+  count: jax.typing.ArrayLike  # shape=(), dtype=jnp.int32.
   mu: base.Updates
   nu: base.Updates
 
 
 def scale_by_belief(
-    b1: float = 0.9,
-    b2: float = 0.999,
-    eps: float = 1e-16,
-    eps_root: float = 1e-16,
+    b1: jax.typing.ArrayLike = 0.9,
+    b2: jax.typing.ArrayLike = 0.999,
+    eps: jax.typing.ArrayLike = 1e-16,
+    eps_root: jax.typing.ArrayLike = 1e-16,
     *,
     nesterov: bool = False,
 ) -> base.GradientTransformation:
@@ -715,11 +755,11 @@ def scale_by_belief(
 
 
 def scale_by_yogi(
-    b1: float = 0.9,
-    b2: float = 0.999,
-    eps: float = 1e-3,
-    eps_root: float = 0.0,
-    initial_accumulator_value: float = 1e-6,
+    b1: jax.typing.ArrayLike = 0.9,
+    b2: jax.typing.ArrayLike = 0.999,
+    eps: jax.typing.ArrayLike = 1e-3,
+    eps_root: jax.typing.ArrayLike = 0.0,
+    initial_accumulator_value: jax.typing.ArrayLike = 1e-6,
 ) -> base.GradientTransformation:
   """Rescale updates according to the Yogi algorithm.
 
@@ -771,11 +811,11 @@ def scale_by_yogi(
 
 
 def scale_by_radam(
-    b1: float = 0.9,
-    b2: float = 0.999,
-    eps: float = 1e-8,
-    eps_root: float = 0.0,
-    threshold: float = 5.0,
+    b1: jax.typing.ArrayLike = 0.9,
+    b2: jax.typing.ArrayLike = 0.999,
+    eps: jax.typing.ArrayLike = 1e-8,
+    eps_root: jax.typing.ArrayLike = 0.0,
+    threshold: jax.typing.ArrayLike = 5.0,
     *,
     nesterov: bool = False,
 ) -> base.GradientTransformation:
@@ -850,11 +890,11 @@ class ScaleByRpropState(NamedTuple):
 
 
 def scale_by_rprop(
-    learning_rate: float,
-    eta_minus: float = 0.5,
-    eta_plus: float = 1.2,
-    min_step_size: float = 1e-6,
-    max_step_size: float = 50.0,
+    learning_rate: jax.typing.ArrayLike,
+    eta_minus: jax.typing.ArrayLike = 0.5,
+    eta_plus: jax.typing.ArrayLike = 1.2,
+    min_step_size: jax.typing.ArrayLike = 1e-6,
+    max_step_size: jax.typing.ArrayLike = 50.0,
 ) -> base.GradientTransformation:
   """Scale with the Rprop optimizer.
 
@@ -936,7 +976,7 @@ def scale_by_sign() -> base.GradientTransformation:
 class ScaleByScheduleState(NamedTuple):
   """Maintains count for scale scheduling."""
 
-  count: chex.Array  # shape=(), dtype=jnp.int32
+  count: jax.typing.ArrayLike  # shape=(), dtype=jnp.int32
 
 
 def scale_by_learning_rate(
@@ -996,9 +1036,9 @@ def scale_by_schedule(
 
 
 def scale_by_trust_ratio(
-    min_norm: float = 0.0,
-    trust_coefficient: float = 1.0,
-    eps: float = 0.0,
+    min_norm: jax.typing.ArrayLike = 0.0,
+    trust_coefficient: jax.typing.ArrayLike = 1.0,
+    eps: jax.typing.ArrayLike = 0.0,
 ) -> base.GradientTransformation:
   """Scale updates by `trust ratio`.
 
@@ -1042,11 +1082,11 @@ def scale_by_trust_ratio(
 class ApplyEvery(NamedTuple):
   """Contains a counter and a gradient accumulator."""
 
-  count: chex.Array
+  count: jax.typing.ArrayLike
   grad_acc: base.Updates
 
 
-def apply_every(k: int = 1) -> base.GradientTransformation:
+def apply_every(k: jax.typing.ArrayLike = 1) -> base.GradientTransformation:
   """Accumulate gradients and apply them every k steps.
 
   Note that if this transformation is part of a chain, the states of the other
@@ -1118,6 +1158,15 @@ def centralize() -> base.GradientTransformation:
   return base.GradientTransformation(base.init_empty_state, update_fn)
 
 
+def _zeros_like_axis(x, axis: int) -> jax.Array:
+  """Returns an array of zeros like the given axis of `x`."""
+  # The logic here is unusual to circumvent sharding-in-types limitations.
+  # We use jax.eval_shape to get the shape/sharding of the given axis of `x`.
+  axes = tuple(i for i in range(x.ndim) if i != axis)
+  target = jax.eval_shape(functools.partial(jnp.sum, axis=axes), x)
+  return jnp.zeros_like(target)
+
+
 class ScaleBySM3State(NamedTuple):
   """State for the SM3 algorithm."""
 
@@ -1126,7 +1175,9 @@ class ScaleBySM3State(NamedTuple):
 
 
 def scale_by_sm3(
-    b1: float = 0.9, b2: float = 1.0, eps: float = 1e-8
+    b1: jax.typing.ArrayLike = 0.9,
+    b2: jax.typing.ArrayLike = 1.0,
+    eps: jax.typing.ArrayLike = 1e-8
 ) -> base.GradientTransformation:
   """Scale updates by `sm3`.
 
@@ -1142,7 +1193,7 @@ def scale_by_sm3(
   """
 
   def zeros_for_dim(p):
-    return [jnp.zeros([s], dtype=p.dtype) for s in p.shape]
+    return [_zeros_like_axis(p, i) for i in range(p.ndim)]
 
   def init_fn(params):
     _reject_complex(params)
@@ -1197,18 +1248,18 @@ def scale_by_sm3(
 class ScaleByNovogradState(NamedTuple):
   """State for Novograd."""
 
-  count: chex.Array
+  count: jax.typing.ArrayLike
   mu: base.Updates
   nu: base.Updates
 
 
 def scale_by_novograd(
-    b1: float = 0.9,
-    b2: float = 0.25,
-    eps: float = 1e-8,
-    eps_root: float = 0.0,
-    weight_decay: float = 0.0,
-    mu_dtype: Optional[chex.ArrayDType] = None,
+    b1: jax.typing.ArrayLike = 0.9,
+    b2: jax.typing.ArrayLike = 0.25,
+    eps: jax.typing.ArrayLike = 1e-8,
+    eps_root: jax.typing.ArrayLike = 0.0,
+    weight_decay: jax.typing.ArrayLike = 0.0,
+    mu_dtype: Optional[jax.typing.DTypeLike] = None,
 ) -> base.GradientTransformation:
   """Computes NovoGrad updates.
 
@@ -1277,17 +1328,17 @@ def scale_by_novograd(
 
 
 class ScaleByOptimisticGradientState(NamedTuple):
-  is_initial_step: chex.Array
+  is_initial_step: jax.typing.ArrayLike
   previous_gradient: base.Updates
 
 
 def scale_by_optimistic_gradient(
-    alpha: float = 1.0, beta: float = 1.0
+    alpha: jax.typing.ArrayLike = 1.0, beta: jax.typing.ArrayLike = 1.0
 ) -> base.GradientTransformation:
   """Compute generalized optimistic gradients.
 
-  See :func:`optax.optimistic_adam`, :func:`optax.optimistic_gradient_descent`
-  for more details.
+  See :func:`optax.optimistic_adam_v2`,
+  :func:`optax.optimistic_gradient_descent` for more details.
 
   Args:
     alpha: Coefficient for generalized optimistic gradient descent.
@@ -1391,7 +1442,7 @@ def scale_by_distance_over_gradients(
       eta = global_scale * (d / jnp.sqrt(g_sos + eps))
       return eta * g
 
-    updates = jax.tree.map(_tx, max_dist, g_sos, updates)
+    updates = jax.tree.map(_tx, updates, max_dist, g_sos)
 
     # new state
     state = ScaleByDistanceOverGradientsState(
@@ -1404,9 +1455,9 @@ def scale_by_distance_over_gradients(
 
 
 def scale_by_polyak(
-    f_min: float = 0.0,
-    max_learning_rate: float = 1.0,
-    eps: float = 0.0,
+    f_min: jax.typing.ArrayLike = 0.0,
+    max_learning_rate: jax.typing.ArrayLike = 1.0,
+    eps: jax.typing.ArrayLike = 0.0,
     variant: str = 'sps',
 ) -> base.GradientTransformationExtraArgs:
   r"""Scales the update by Polyak's step-size.
@@ -1431,7 +1482,7 @@ def scale_by_polyak(
       state: base.EmptyState,
       params: Optional[base.Params] = None,
       *,
-      value: float,
+      value: jax.typing.ArrayLike,
       **extra_args,
   ) -> tuple[base.Updates, base.EmptyState]:
     """Scales the update by the Polyak step-size.
@@ -1447,8 +1498,9 @@ def scale_by_polyak(
       The scaled updates and the state of the transformation.
     """
     del params
-    del extra_args  # complies with signature of GradientTransformationExtraArgs
-                    # but ignores the extra_args
+    # complies with signature of GradientTransformationExtraArgs but ignores the
+    # extra_args
+    del extra_args
     grad_sq_norm = optax.tree.norm(updates, squared=True)
     gap = jnp.array(value - f_min).astype(grad_sq_norm.dtype)
     if variant == 'sps':
@@ -1486,21 +1538,21 @@ class ScaleByLBFGSState(NamedTuple):
       :func:`optax.scale_by_lbfgs` for more details.
   """
 
-  count: chex.Numeric
+  count: jax.typing.ArrayLike
   params: base.Params
   updates: base.Params
-  diff_params_memory: chex.ArrayTree
-  diff_updates_memory: chex.ArrayTree
-  weights_memory: chex.Array
+  diff_params_memory: base.ArrayTree
+  diff_updates_memory: base.ArrayTree
+  weights_memory: jax.typing.ArrayLike
 
 
 def _precondition_by_lbfgs(
     updates: base.Updates,
-    diff_params_memory: chex.ArrayTree,
-    diff_updates_memory: chex.ArrayTree,
-    weights_memory: chex.Array,
-    identity_scale: Union[float, jax.Array],
-    memory_idx: Union[int, jax.Array],
+    diff_params_memory: base.ArrayTree,
+    diff_updates_memory: base.ArrayTree,
+    weights_memory: jax.typing.ArrayLike,
+    identity_scale: jax.typing.ArrayLike,  # float
+    memory_idx: jax.typing.ArrayLike,  # int
 ) -> base.Updates:
   r"""Multiplies updates by an approximation of the inverse Hessian.
 
@@ -1615,7 +1667,7 @@ def scale_by_lbfgs(
   preconditioning matrix subject to some secant condition, see references
   for more details. Computing :math:`P_k u_k` can be done by a sequence of
   vector operations using past differences of parameters and gradients stored in
-  a memory bufffer.
+  a memory buffer.
 
   The present function just outputs the LBFGS direction :math:`P_k u_k`.
   It can be chained with a linesearch ensuring sufficient decrease and low
@@ -1635,12 +1687,12 @@ def scale_by_lbfgs(
 
   References:
     Algorithms 7.4, 7.5 (page 199) of Nocedal et al, `Numerical Optimization
-    <https://www.math.uci.edu/~qnie/Publications/NumericalOptimization.pdf>`__
-    , 1999
+    <https://www.math.uci.edu/~qnie/Publications/NumericalOptimization.pdf>`_,
+    1999
 
     Liu et al., `On the limited memory BFGS method for large scale optimization
-    <https://users.iems.northwestern.edu/~nocedal/PDFfiles/limited-memory.pdf>`_
-    , 1989.
+    <https://users.iems.northwestern.edu/~nocedal/PDFfiles/limited-memory.pdf>`_,
+    1989.
 
   .. note::
     We initialize the scaling of the identity as a capped reciprocal of the
@@ -1754,7 +1806,7 @@ def scale_by_lbfgs(
 
 
 def normalize_by_update_norm(
-    scale_factor: float = 1.0, eps: float = 1e-6
+    scale_factor: jax.typing.ArrayLike = 1.0, eps: jax.typing.ArrayLike = 1e-6
 ) -> base.GradientTransformation:
   """Scale by the inverse of the update norm.
 
@@ -1800,17 +1852,7 @@ def normalize_by_update_norm(
   return base.GradientTransformation(base.init_empty_state, update_fn)
 
 
-### Legacy symbols to be removed. ###
-
-
-@functools.partial(
-    chex.warn_deprecated_function, replacement='optax.tree.cast'
-)
-def cast_tree(
-    tree: chex.ArrayTree, dtype: Optional[chex.ArrayDType]
-) -> chex.ArrayTree:
-  return optax.tree.cast(tree, dtype)
-
+# Legacy symbols to be removed.
 
 trace = _accumulation.trace
 TraceState = _accumulation.TraceState
