@@ -20,10 +20,10 @@ by Keller Jordan
 """
 
 
+import functools
 import math
 from typing import Any, Callable, NamedTuple, Optional, Union, Sequence
 
-import chex
 import jax
 import jax.numpy as jnp
 
@@ -39,6 +39,9 @@ import optax.tree
 ReshapeFn = Callable[[jax.Array], jax.Array]
 
 
+_DEFAULT_NS_COEFFS = (3.4445, -4.7750, 2.0315)
+
+
 class MuonDimensionNumbers(NamedTuple):
   """Specification for which weight axes participate in matrix projection.
 
@@ -60,6 +63,7 @@ class MuonDimensionNumbers(NamedTuple):
   reduction_axis: Sequence[int] | int = 0
   output_axis: Sequence[int] | int = 1
 
+
 WeightDimNumOrFn = MuonDimensionNumbers | base.Params | Callable[
     [base.Params], base.Params | None]
 
@@ -113,19 +117,89 @@ def _compute_muon_reshape(x: jax.Array, dim_nums: MuonDimensionNumbers
   return reshape_fn, inverse_fn
 
 
-def _shape_factor(x: jax.Array, dim_nums: MuonDimensionNumbers) -> float:
+def _get_shape_products(
+    x: jax.Array, dim_nums: MuonDimensionNumbers
+) -> tuple[float, float]:
   reduction_axes, output_axes = _normalize_axes(x, dim_nums)
-  return math.prod(x.shape[ax] for ax in output_axes) / math.prod(
-      x.shape[ax] for ax in reduction_axes)
+  fan_in = math.prod(x.shape[ax] for ax in reduction_axes)
+  fan_out = math.prod(x.shape[ax] for ax in output_axes)
+  return fan_in, fan_out
+
+
+def _scale_update_for_width_transfer(
+    update: jax.Array, dim_nums: MuonDimensionNumbers
+):
+  """Apply width scaling from <https://github.com/KellerJordan/Muon>."""
+  fan_in, fan_out = _get_shape_products(update, dim_nums)
+  scale = jnp.sqrt(jnp.maximum(1, fan_out / fan_in))
+  return scale * update
+
+
+def _scale_update_for_consistent_rms(
+    update: jax.Array,
+    dim_nums: MuonDimensionNumbers,
+    consistent_rms: jax.typing.ArrayLike
+):
+  """Apply consistent RMS scaling from <https://arxiv.org/abs/2502.16982>."""
+  fan_in, fan_out = _get_shape_products(update, dim_nums)
+  scale = jnp.sqrt(jnp.maximum(fan_in, fan_out)) * consistent_rms
+  return scale * update
+
+
+def scale_by_shape(
+    weight_dimension_numbers: WeightDimNumOrFn | None = None,
+    consistent_rms: jax.typing.ArrayLike | None = None,
+) -> base.GradientTransformation:
+  """Scale updates by factors derived from parameter shape.
+
+  Args:
+    weight_dimension_numbers: An optional tree with the same structure as the
+      params of `MuonDimensionNumbers`s, specifying how to reshape the
+      parameters before and after the orthogonalization OR a callable returning
+      such a tree. None implies that all parameters are 2D matrices.
+    consistent_rms: An optional float to activate consistent RMS scaling.
+      If float, scales updates by `sqrt(max(fan_in, fan_out)) * consistent_rms`.
+      If None, uses width scaling `sqrt(max(1, fan_out / fan_in))`.
+
+  Returns:
+    A `GradientTransformation` object.
+  """
+
+  def update_fn(updates, state, params=None):
+    del params
+    if callable(weight_dimension_numbers):
+      # Populate weight_dim_nums if it's a callable. Use updates instead of
+      # actual params since only shapes matter and params may not be provided.
+      resolved_weight_dim_nums = weight_dimension_numbers(updates)
+    else:
+      resolved_weight_dim_nums = weight_dimension_numbers
+
+    if consistent_rms is not None:
+      scaling_fn = functools.partial(
+          _scale_update_for_consistent_rms, consistent_rms=consistent_rms
+      )
+    else:
+      scaling_fn = _scale_update_for_width_transfer
+
+    scaled_updates = jax.tree.map(
+        scaling_fn,
+        updates,
+        resolved_weight_dim_nums,
+        is_leaf=_is_weight_dim_nums,
+    )
+    return scaled_updates, state
+
+  # Use the standard empty_state initializer, as this transform is stateless
+  return base.GradientTransformation(base.init_empty_state, update_fn)
 
 
 def _newton_schulz_iterator(x: jax.Array, coeffs: jax.Array) -> jax.Array:
   # Implements Newton-Schulz step f(X) = c_0 X + c_1 (XX^T)X + c_2 (XX^T)^2X,
   # with quintic form f(X) = c_0 X + (c_1 A + c_2 AA)X, where A = XX^T.
   # The NS step has the property f(X) = f(X^T)^T. That is, we can get equivalent
-  # result by tranposing input and output. In particular, we may tranpose X
-  # when rows > cols for effciency.
-  a = x @ x.T
+  # result by transposing input and output. In particular, we may transpose X
+  # when rows > cols for efficiency.
+  a = x @ x.T.conj()
   b = coeffs[1] * a + coeffs[2] * a @ a
   return coeffs[0] * x + b @ x
 
@@ -133,8 +207,8 @@ def _newton_schulz_iterator(x: jax.Array, coeffs: jax.Array) -> jax.Array:
 def orthogonalize_via_newton_schulz(
     x: jax.Array,
     ns_coeffs: jax.Array,
-    ns_steps: int = 5,
-    eps: float = 1e-8,
+    ns_steps: jax.typing.ArrayLike = 5,
+    eps: jax.typing.ArrayLike = 1e-8,
     dimension_numbers: MuonDimensionNumbers | None = None,
 ) -> jax.Array:
   r"""Orthogonalize via Newton-Schulz iteration.
@@ -197,20 +271,21 @@ def orthogonalize_via_newton_schulz(
 
 class MuonState(NamedTuple):
   """State for the Muon algorithm."""
-  count: chex.Array  # shape=(), dtype=jnp.int32.
+  count: jax.typing.ArrayLike  # shape=(), dtype=jnp.int32.
   mu: base.Updates
-  ns_coeffs: chex.Array  # shape=(), dtype=jnp.int32.
+  ns_coeffs: jax.typing.ArrayLike  # shape=(), dtype=jnp.int32.
 
 
 def scale_by_muon(
     ns_coeffs: Union[
-        tuple[float, float, float],
-        tuple[tuple[float, float, float], ...],
-    ] = (3.4445, -4.7750, 2.0315),
-    ns_steps: int = 5,
-    beta: float = 0.95,
-    eps: float = 1e-8,
-    mu_dtype: Optional[chex.ArrayDType] = None,
+        tuple[jax.typing.ArrayLike, jax.typing.ArrayLike, jax.typing.ArrayLike],
+        tuple[tuple[jax.typing.ArrayLike, jax.typing.ArrayLike,
+                    jax.typing.ArrayLike], ...],
+    ] = _DEFAULT_NS_COEFFS,
+    ns_steps: jax.typing.ArrayLike = 5,
+    beta: jax.typing.ArrayLike = 0.95,
+    eps: jax.typing.ArrayLike = 1e-8,
+    mu_dtype: Optional[jax.typing.DTypeLike] = None,
     *,
     nesterov: bool = True,
     adaptive: bool = False,
@@ -236,7 +311,7 @@ def scale_by_muon(
       original updates. See <https://arxiv.org/abs/2409.20325>
     weight_dimension_numbers: An optional tree with the same structure as the
       params of `MuonDimensionNumbers`s, specifying how to reshape the
-      parameters before and after the orthogonalization OR a callble returning
+      parameters before and after the orthogonalization OR a callable returning
       such a tree. None implies that all parameters are 2D matrices.
 
   Returns:
@@ -295,14 +370,9 @@ def scale_by_muon(
       # Scale the orthogonalized updates by the dual norm of the original
       # updates. See https://arxiv.org/abs/2409.20325 for the derivation.
       updates = jax.tree.map(
-          lambda x, y: jnp.sum(x * y) * y, mu_hat, updates
+          lambda x, y: jnp.sum(x.conj() * y) * y, mu_hat, updates
       )
-    factors = jax.tree.map(_shape_factor, updates, resolved_weight_dim_nums,
-                           is_leaf=_is_weight_dim_nums)
-    updates = jax.tree.map(
-        lambda x, factor: jnp.sqrt(jnp.maximum(1, factor)) * x,
-        updates, factors
-    )
+
     mu = optax.tree.cast(mu, mu_dtype)
     return updates, MuonState(
         count=count_inc,
@@ -315,25 +385,28 @@ def scale_by_muon(
 def muon(
     learning_rate: base.ScalarOrSchedule,
     ns_coeffs: Union[
-        tuple[float, float, float],
-        tuple[tuple[float, float, float], ...],
-    ] = (3.4445, -4.7750, 2.0315),
-    ns_steps: int = 5,
-    beta: float = 0.95,
-    eps: float = 1e-8,
-    weight_decay: float = 0.0,
+        tuple[jax.typing.ArrayLike, jax.typing.ArrayLike, jax.typing.ArrayLike],
+        tuple[tuple[jax.typing.ArrayLike, jax.typing.ArrayLike,
+                    jax.typing.ArrayLike], ...],
+    ] = _DEFAULT_NS_COEFFS,
+    ns_steps: jax.typing.ArrayLike = 5,
+    beta: jax.typing.ArrayLike = 0.95,
+    eps: jax.typing.ArrayLike = 1e-8,
+    weight_decay: jax.typing.ArrayLike = 0.0,
     weight_decay_mask: Optional[
         Union[Any, Callable[[base.Params], Any]]
     ] = None,
-    mu_dtype: Optional[chex.ArrayDType] = None,
+    mu_dtype: Optional[jax.typing.DTypeLike] = None,
     *,
     nesterov: bool = True,
     adaptive: bool = False,
-    adam_b1: float = 0.9,
-    adam_b2: float = 0.999,
-    adam_eps_root: float = 0.0,
-    adam_weight_decay: float = 0.0,
+    adam_b1: jax.typing.ArrayLike = 0.9,
+    adam_b2: jax.typing.ArrayLike = 0.999,
+    adam_eps_root: jax.typing.ArrayLike = 0.0,
+    adam_weight_decay: jax.typing.ArrayLike = 0.0,
+    adam_learning_rate: base.ScalarOrSchedule | None = None,
     muon_weight_dimension_numbers: WeightDimNumOrFn | None = None,
+    consistent_rms: jax.typing.ArrayLike | None = None,
 ) -> base.GradientTransformation:
   r"""Muon: Momentum Orthogonalized by Newton-schulz.
 
@@ -345,7 +418,8 @@ def muon(
 
   Note that Muon is currently only defined for 2D parameters, i.e. matrices.
   This is because the Newton-Schulz iterator expects a matrix as input.
-  The non-2D parameters are instead passed through an Adam optimizer.
+  The non-2D parameters are instead passed through an AdamW optimizer
+  (using a weight decay of 0 as default).
 
   Args:
     learning_rate: A global scaling factor, either fixed or evolving along
@@ -372,6 +446,8 @@ def muon(
     adam_b2: Exponential decay rate for Adam's second moment estimates.
     adam_eps_root: Epsilon to stabilize division in Adam, square root version.
     adam_weight_decay: Weight decay factor for Adam.
+    adam_learning_rate: Auxilary learning rate for the Adam optimizer.
+      If `None`, the learning rate for Adam defaults to the same as Muon.
     muon_weight_dimension_numbers: An optional tree of `MuonDimensionNumbers`s,
       specifying how to reshape the parameters for orthogonalization otherwise
       muon parameters are assumed to be 2D matrices. A `None` value indicates
@@ -379,6 +455,11 @@ def muon(
       Adam. A callable takes as input the params and returns a possibly masked
       pytree of specs, similar to `weight_decay_mask`. If not provided, muon is
       applied to all 2D parameters.
+    consistent_rms: An optional float to activate consistent RMS scaling.
+      Scales updates by `sqrt(max(fan_in, fan_out)) * consistent_rms` to make
+      root mean square (RMS) shape-independent, like AdamW. `0.2` is recommended
+      to match AdamW's empirical RMS. See <https://arxiv.org/abs/2502.16982>.
+      If `None`, uses width scaling `sqrt(max(1, fan_out / fan_in))`.
 
   Returns:
     The corresponding `GradientTransformation`.
@@ -389,7 +470,14 @@ def muon(
 
     Bernstein et al., `Old Optimizer, New Norm: An Anthology
     <https://arxiv.org/abs/2409.20325>`_, 2024
+
+    Liu et al., `Muon is Scalable for LLM Training`,
+    <https://arxiv.org/abs/2502.16982>`_, 2025
   """
+
+  if adam_learning_rate is None:
+    adam_learning_rate = learning_rate
+
   # None at root indicates the default 2D rule.
   if muon_weight_dimension_numbers is None:
     param_labels = lambda params: jax.tree.map(
@@ -437,11 +525,15 @@ def muon(
                   adaptive=adaptive,
                   weight_dimension_numbers=muon_weight_dim_nums_fn,
               ),
+              scale_by_shape(
+                  weight_dimension_numbers=muon_weight_dim_nums_fn,
+                  consistent_rms=consistent_rms,
+              ),
               transform.add_decayed_weights(weight_decay, weight_decay_mask),
               transform.scale_by_learning_rate(learning_rate),
           ),
           'adam': alias.adamw(
-              learning_rate=learning_rate,
+              learning_rate=adam_learning_rate,
               b1=adam_b1,
               b2=adam_b2,
               eps=eps,
