@@ -14,14 +14,20 @@
 # ==============================================================================
 """Smoothing functions."""
 
-import chex
+from typing import Union
+
+import jax
 import jax.numpy as jnp
+from optax._src import utils
 
 
 def smooth_labels(
-    labels: chex.Array,
-    alpha: float,
-) -> jnp.ndarray:
+    labels: jax.typing.ArrayLike,
+    alpha: jax.typing.ArrayLike,
+    *,
+    axis: Union[int, tuple[int, ...], None] = -1,
+    where: Union[jax.typing.ArrayLike, None] = None,
+) -> jax.Array:
   """Apply label smoothing.
 
   Label smoothing is often used in combination with a cross-entropy loss.
@@ -31,6 +37,8 @@ def smooth_labels(
   Args:
     labels: One hot labels to be smoothed.
     alpha: The smoothing factor.
+    axis: Axis or axes along which to compute.
+    where: Elements to include in the computation.
 
   Returns:
     a smoothed version of the one hot input labels.
@@ -39,6 +47,9 @@ def smooth_labels(
     Muller et al, `When does label smoothing help?
     <https://arxiv.org/abs/1906.02629>`_, 2019
   """
-  chex.assert_type([labels], float)
-  num_categories = labels.shape[-1]
+  utils.check_subdtype(labels, jnp.floating)
+  if where is None:
+    num_categories = jnp.size(labels, axis)
+  else:
+    num_categories = jnp.sum(where, axis, keepdims=True)
   return (1.0 - alpha) * labels + alpha / num_categories
