@@ -12,10 +12,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-"""Tests for optax."""
+"""Tests for importing optax."""
 
 from absl.testing import absltest
-
 import optax
 from optax import transforms
 
