@@ -19,18 +19,16 @@ instance, they may be used to anneal the learning rate used to update an agent's
 parameters or the exploration factor used to select actions.
 """
 
-from typing import Iterable, Optional, Union
+from typing import Iterable, Optional
 
 from absl import logging
-import chex
 import jax
 import jax.numpy as jnp
-import numpy as np
 from optax._src import base
 from optax.schedules import _join
 
 
-def constant_schedule(value: Union[float, int]) -> base.Schedule:
+def constant_schedule(value: jax.typing.ArrayLike) -> base.Schedule:
   """Constructs a constant schedule.
 
   Args:
@@ -51,9 +49,9 @@ def constant_schedule(value: Union[float, int]) -> base.Schedule:
 
 
 def polynomial_schedule(
-    init_value: chex.Scalar,
-    end_value: chex.Scalar,
-    power: chex.Scalar,
+    init_value: jax.typing.ArrayLike,
+    end_value: jax.typing.ArrayLike,
+    power: jax.typing.ArrayLike,
     transition_steps: int,
     transition_begin: int = 0,
 ) -> base.Schedule:
@@ -148,8 +146,8 @@ def polynomial_schedule(
 
 
 def linear_schedule(
-    init_value: chex.Scalar,
-    end_value: chex.Scalar,
+    init_value: jax.typing.ArrayLike,
+    end_value: jax.typing.ArrayLike,
     transition_steps: int,
     transition_begin: int = 0,
 ) -> base.Schedule:
@@ -203,7 +201,8 @@ def linear_schedule(
 
 
 def piecewise_constant_schedule(
-    init_value: float, boundaries_and_scales: Optional[dict[int, float]] = None
+    init_value: jax.typing.ArrayLike,  # float
+    boundaries_and_scales: Optional[dict[int, float]] = None
 ) -> base.Schedule:
   """Piecewise constant schedule with scaled jumps at specific boundaries.
 
@@ -249,12 +248,12 @@ def piecewise_constant_schedule(
 
 
 def exponential_decay(
-    init_value: float,
+    init_value: jax.typing.ArrayLike,
     transition_steps: int,
     decay_rate: float,
     transition_begin: int = 0,
     staircase: bool = False,
-    end_value: Optional[float] = None,
+    end_value: Optional[jax.typing.ArrayLike] = None,
 ) -> base.Schedule:
   """Constructs a schedule with either continuous or discrete exponential decay.
 
@@ -327,10 +326,10 @@ def exponential_decay(
 
 
 def cosine_decay_schedule(
-    init_value: float,
+    init_value: jax.typing.ArrayLike,
     decay_steps: int,
-    alpha: float = 0.0,
-    exponent: float = 1.0,
+    alpha: jax.typing.ArrayLike = 0.0,
+    exponent: jax.typing.ArrayLike = 1.0,
 ) -> base.Schedule:
   r"""Returns a function which implements cosine learning rate decay.
 
@@ -390,19 +389,25 @@ def cosine_decay_schedule(
   return schedule
 
 
-def _linear_interpolate(start: float, end: float, pct: float):
+def _linear_interpolate(
+    start: jax.typing.ArrayLike,
+    end: jax.typing.ArrayLike,
+    pct: jax.typing.ArrayLike):
   """Linearly interpolate between two values."""
   return (end - start) * pct + start
 
 
-def _cosine_interpolate(start: float, end: float, pct: float):
+def _cosine_interpolate(
+    start: jax.typing.ArrayLike,
+    end: jax.typing.ArrayLike,
+    pct: jax.typing.ArrayLike):
   """Cosine interpolate between two values (smoother transitions)."""
   return end + (start - end) / 2.0 * (jnp.cos(jnp.pi * pct) + 1)
 
 
 def piecewise_interpolate_schedule(
     interpolate_type: str,
-    init_value: float,
+    init_value: jax.typing.ArrayLike,
     boundaries_and_scales: Optional[dict[int, float]] = None,
 ) -> base.Schedule:
   """Piecewise interpolated schedule with linear or cosine transitions.
@@ -463,12 +468,11 @@ def piecewise_interpolate_schedule(
   else:
     boundaries, scales = (), ()
 
-  bounds = np.stack((0,) + boundaries)
-  values = np.cumprod(np.stack((init_value,) + scales))
-  interval_sizes = bounds[1:] - bounds[:-1]
-
   def schedule(count):
-    indicator = (bounds[:-1] <= count) & (count < bounds[1:])
+    bounds = jnp.stack((0,) + boundaries)
+    values = jnp.cumprod(jnp.stack((init_value,) + scales))
+    interval_sizes = jnp.maximum(bounds[1:] - bounds[:-1], 1)
+    indicator = jnp.logical_and(bounds[:-1] <= count, count < bounds[1:])
     pct = (count - bounds[:-1]) / interval_sizes
     interp_vals = interpolate_fn(values[:-1], values[1:], pct)
     return indicator.dot(interp_vals) + (bounds[-1] <= count) * values[-1]
@@ -478,7 +482,7 @@ def piecewise_interpolate_schedule(
 
 def linear_onecycle_schedule(
     transition_steps: int,
-    peak_value: float,
+    peak_value: jax.typing.ArrayLike,  # float
     pct_start: float = 0.3,
     pct_final: float = 0.85,
     div_factor: float = 25.0,
@@ -537,7 +541,7 @@ def linear_onecycle_schedule(
 
 def cosine_onecycle_schedule(
     transition_steps: int,
-    peak_value: float,
+    peak_value: jax.typing.ArrayLike,  # float
     pct_start: float = 0.3,
     div_factor: float = 25.0,
     final_div_factor: float = 1e4,
@@ -588,8 +592,8 @@ def cosine_onecycle_schedule(
 
 
 def warmup_constant_schedule(
-    init_value: float,
-    peak_value: float,
+    init_value: jax.typing.ArrayLike,
+    peak_value: jax.typing.ArrayLike,
     warmup_steps: int,
 ) -> base.Schedule:
   r"""Linear warmup followed by constant schedule i.e no decay.
@@ -611,12 +615,12 @@ def warmup_constant_schedule(
 
 
 def warmup_cosine_decay_schedule(
-    init_value: float,
-    peak_value: float,
+    init_value: jax.typing.ArrayLike,
+    peak_value: jax.typing.ArrayLike,
     warmup_steps: int,
     decay_steps: int,
-    end_value: float = 0.0,
-    exponent: float = 1.0,
+    end_value: jax.typing.ArrayLike = 0.0,
+    exponent: jax.typing.ArrayLike = 1.0,
 ) -> base.Schedule:
   r"""Linear warmup followed by cosine decay.
 
@@ -654,14 +658,14 @@ def warmup_cosine_decay_schedule(
 
 
 def warmup_exponential_decay_schedule(
-    init_value: float,
-    peak_value: float,
+    init_value: jax.typing.ArrayLike,
+    peak_value: jax.typing.ArrayLike,
     warmup_steps: int,
     transition_steps: int,
-    decay_rate: float,
+    decay_rate: jax.typing.ArrayLike,
     transition_begin: int = 0,
     staircase: bool = False,
-    end_value: Optional[float] = None,
+    end_value: Optional[jax.typing.ArrayLike] = None,
 ) -> base.Schedule:
   """Linear warmup followed by exponential decay.
 
@@ -669,8 +673,8 @@ def warmup_exponential_decay_schedule(
     init_value: Initial value for the scalar to be annealed.
     peak_value: Peak value for scalar to be annealed at end of warmup.
     warmup_steps: Positive integer, the length of the linear warmup.
-    transition_steps: must be positive. See :func:`optax.exponential_decay` for
-      more details.
+    transition_steps: must be positive.
+      See :func:`optax.schedules.exponential_decay` for more details.
     decay_rate: must not be zero. The decay rate.
     transition_begin: must be positive. After how many steps to start annealing
       (before this many steps the scalar value is held fixed at ``peak_value``).
@@ -702,7 +706,7 @@ def warmup_exponential_decay_schedule(
 
 
 def sgdr_schedule(
-    cosine_kwargs: Iterable[dict[str, chex.Numeric]],
+    cosine_kwargs: Iterable[dict[str, jax.typing.ArrayLike]],
 ) -> base.Schedule:
   """SGD with warm restarts.
 
