@@ -17,14 +17,14 @@
 import functools
 from typing import Optional, TypeVar
 
-import chex
 import jax
 import jax.numpy as jnp
+from optax._src import base
 
 T = TypeVar('T')
 
 
-def tree_cast_like(tree: T, other_tree: chex.ArrayTree) -> T:
+def tree_cast_like(tree: T, other_tree: base.ArrayTree) -> T:
   """Cast tree to dtypes of other_tree.
 
   Args:
@@ -49,8 +49,8 @@ def tree_cast_like(tree: T, other_tree: chex.ArrayTree) -> T:
 
 
 def tree_cast(
-    tree: chex.ArrayTree, dtype: Optional[chex.ArrayDType]
-) -> chex.ArrayTree:
+    tree: base.ArrayTree, dtype: Optional[jax.typing.DTypeLike]
+) -> base.ArrayTree:
   """Cast tree to given dtype, skip if None.
 
   Args:
@@ -75,8 +75,8 @@ def tree_cast(
 
 
 def tree_dtype(
-    tree: chex.ArrayTree, mixed_dtype_handler: Optional[str] = None
-) -> chex.ArrayDType:
+    tree: base.ArrayTree, mixed_dtype_handler: Optional[str] = None
+) -> jax.typing.DTypeLike:
   """Fetch dtype of tree.
 
   If the tree is empty, returns the default dtype of JAX arrays.
@@ -127,7 +127,7 @@ def tree_dtype(
     >>> # -> will throw an error because int32 and uint32
     >>> # cannot be promoted to one another.
     >>> optax.tree_utils.tree_dtype(tree, 'promote')
-    dtype('int64')
+    dtype('int32')
 
   .. seealso:: :func:`jax.numpy.promote_types`,
     `Type promotion semantics in JAX
@@ -139,7 +139,7 @@ def tree_dtype(
   if not leaves:
     # If the tree is empty, we return the default dtype as given by JAX on
     # empty lists.
-    return jnp.dtype(jnp.asarray(leaves))
+    return jnp.asarray(leaves).dtype
   if mixed_dtype_handler is None:
     dtype = jnp.asarray(leaves[0]).dtype
     _tree_assert_all_dtypes_equal(tree, dtype)
@@ -166,7 +166,7 @@ def tree_dtype(
 
 
 def _tree_assert_all_dtypes_equal(
-    tree: chex.ArrayTree, dtype: chex.ArrayDType
+    tree: base.ArrayTree, dtype: jax.typing.DTypeLike
 ) -> None:
   """Checks that all leaves of the tree have the given dtype.
 
@@ -194,8 +194,8 @@ def _tree_assert_all_dtypes_equal(
 
 
 def _lower_dtype(
-    dtype1: chex.ArrayDType, dtype2: chex.ArrayDType
-) -> chex.ArrayDType:
+    dtype1: jax.typing.DTypeLike, dtype2: jax.typing.DTypeLike
+) -> jax.typing.DTypeLike:
   """Returns lower dtype among two dtypes, if any can be promoted to the other.
 
   Args:
@@ -219,8 +219,8 @@ def _lower_dtype(
 
 
 def _higher_dtype(
-    dtype1: chex.ArrayDType, dtype2: chex.ArrayDType
-) -> chex.ArrayDType:
+    dtype1: jax.typing.DTypeLike, dtype2: jax.typing.DTypeLike
+) -> jax.typing.DTypeLike:
   """Returns higher dtype among two dtypes, if any can be promoted to the other.
 
   Args:
