@@ -16,12 +16,12 @@
 
 from absl.testing import absltest
 from absl.testing import parameterized
-import chex
 import jax
 from jax import flatten_util
 import jax.numpy as jnp
 import numpy as np
 from optax import tree_utils as tu
+from optax._src import test_utils
 
 
 class TreeUtilsTest(parameterized.TestCase):
@@ -63,7 +63,7 @@ class TreeUtilsTest(parameterized.TestCase):
         self.tree_a[1] + self.tree_b[1],
     )
     got = tu.tree_add(self.tree_a, self.tree_b)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_tree_sub(self):
     expected = self.array_a - self.array_b
@@ -75,7 +75,7 @@ class TreeUtilsTest(parameterized.TestCase):
         self.tree_a[1] - self.tree_b[1],
     )
     got = tu.tree_sub(self.tree_a, self.tree_b)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_tree_mul(self):
     expected = self.array_a * self.array_b
@@ -87,7 +87,7 @@ class TreeUtilsTest(parameterized.TestCase):
         self.tree_a[1] * self.tree_b[1],
     )
     got = tu.tree_mul(self.tree_a, self.tree_b)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_tree_div(self):
     expected = self.array_a / self.array_b
@@ -99,7 +99,7 @@ class TreeUtilsTest(parameterized.TestCase):
         self.tree_a[1] / self.tree_b[1],
     )
     got = tu.tree_div(self.tree_a, self.tree_b)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_tree_scale(self):
     expected = 0.5 * self.array_a
@@ -108,7 +108,7 @@ class TreeUtilsTest(parameterized.TestCase):
 
     expected = (0.5 * self.tree_a[0], 0.5 * self.tree_a[1])
     got = tu.tree_scale(0.5, self.tree_a)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_tree_add_scale(self):
     expected = (
@@ -116,12 +116,12 @@ class TreeUtilsTest(parameterized.TestCase):
         self.tree_a[1] + 0.5 * self.tree_b[1],
     )
     got = tu.tree_add_scale(self.tree_a, 0.5, self.tree_b)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_tree_add_scale_dtype(self):
     got = tu.tree_add_scale(3, 0.1, 2)
     expected = 3 + 0.1 * 2
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_tree_vdot(self):
     expected = jnp.vdot(self.array_a, self.array_b)
@@ -136,7 +136,7 @@ class TreeUtilsTest(parameterized.TestCase):
         self.tree_a[1], self.tree_b[1]
     )
     got = tu.tree_vdot(self.tree_a, self.tree_b)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_tree_sum(self):
     expected = jnp.sum(self.array_a)
@@ -147,6 +147,15 @@ class TreeUtilsTest(parameterized.TestCase):
     got = tu.tree_sum(self.tree_a)
     np.testing.assert_allclose(expected, got)
 
+    # Test that associative version matches regular version
+    if hasattr(jax.tree, 'reduce_associative'):
+      with self.subTest('associative_version_matches_regular'):
+        regular_result = tu.tree_sum(self.tree_a)
+        associative_result = tu.tree_sum(
+            self.tree_a, associative_reduction=True
+        )
+        np.testing.assert_allclose(regular_result, associative_result)
+
   @parameterized.parameters(
       'array_a', 'tree_a', 'tree_a_dict', 'tree_b', 'tree_b_dict'
   )
@@ -167,6 +176,22 @@ class TreeUtilsTest(parameterized.TestCase):
     got = tu.tree_min(tree)
     np.testing.assert_allclose(expected, got)
 
+  def test_tree_min_empty(self):
+    tree = [jnp.ones([2, 3]), jnp.zeros([4, 0, 5])]
+    got = tu.tree_min(tree)
+    expected = 1.0
+    assert expected == got
+
+  @parameterized.product(
+      expected=(0, 10000, -10000, float('inf'), -float('inf')),
+      dtype=('int8', 'uint8', 'float32'),
+      which=(tu.tree_min, tu.tree_max),
+  )
+  def test_tree_max_min_empty_dtype(self, expected, dtype, which):
+    tree = [expected, jnp.zeros(0, dtype)]
+    got = which(tree)
+    assert expected == got
+
   @parameterized.parameters(
       'array_a', 'tree_a', 'tree_a_dict', 'tree_b', 'tree_b_dict'
   )
@@ -189,7 +214,7 @@ class TreeUtilsTest(parameterized.TestCase):
 
     expected = (jnp.conj(self.tree_a[0]), jnp.conj(self.tree_a[1]))
     got = tu.tree_conj(self.tree_a)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_tree_real(self):
     expected = jnp.real(self.array_a)
@@ -198,7 +223,7 @@ class TreeUtilsTest(parameterized.TestCase):
 
     expected = (jnp.real(self.tree_a[0]), jnp.real(self.tree_a[1]))
     got = tu.tree_real(self.tree_a)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_tree_l2_norm(self):
     expected = jnp.sqrt(jnp.vdot(self.array_a, self.array_a).real)
@@ -239,7 +264,7 @@ class TreeUtilsTest(parameterized.TestCase):
 
     expected = (jnp.zeros_like(self.tree_a[0]), jnp.zeros_like(self.tree_a[1]))
     got = tu.tree_zeros_like(self.tree_a)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_tree_ones_like(self):
     expected = jnp.ones_like(self.array_a)
@@ -248,26 +273,26 @@ class TreeUtilsTest(parameterized.TestCase):
 
     expected = (jnp.ones_like(self.tree_a[0]), jnp.ones_like(self.tree_a[1]))
     got = tu.tree_ones_like(self.tree_a)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_add_multiple_trees(self):
     """Test adding more than 2 trees with tree_add."""
     trees = [self.tree_a_dict_jax, self.tree_a_dict_jax, self.tree_a_dict_jax]
     expected = tu.tree_scale(3.0, self.tree_a_dict_jax)
     got = tu.tree_add(*trees)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_tree_clip(self):
     """Clip the tree to range [min_value, max_value]."""
     expected = tu.tree_scale(0.5, self.tree_a_dict_jax)
     got = tu.tree_clip(self.tree_a_dict_jax, min_value=0, max_value=0.5)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
     expected = tu.tree_scale(0.5, self.tree_a_dict_jax)
     got = tu.tree_clip(self.tree_a_dict_jax, min_value=None, max_value=0.5)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
     expected = tu.tree_scale(2.0, self.tree_a_dict_jax)
     got = tu.tree_clip(self.tree_a_dict_jax, min_value=2.0, max_value=None)
-    chex.assert_trees_all_close(expected, got)
+    test_utils.assert_trees_all_close(expected, got)
 
   def test_update_infinity_moment(self):
     values = jnp.array([5.0, 7.0])
@@ -305,9 +330,9 @@ class TreeUtilsTest(parameterized.TestCase):
     m = jnp.logspace(-10, 10, num=21, dtype=jnp.bfloat16)  # 1e-10 ... 1e10
     for decay in (0.9, 0.99, 0.999, 0.9995):
       for count in (1, 10, 100, 1000):
-        chex.assert_tree_all_finite(
+        test_utils.assert_tree_all_finite(
             tu.tree_bias_correction(m, decay, count),
-            custom_message=f'failed with decay={decay}, count={count}',
+            err_msg=f'failed with decay={decay}, count={count}',
         )
 
   def test_empty_tree_reduce(self):
